@@ -1,9 +1,9 @@
-/* $Id: color.c 5166 2015-03-27 13:46:50Z bens $ */
+/* $Id: color.c 5249 2015-06-14 19:14:41Z bens $ */
 /**************************************************************************
  *   color.c                                                              *
  *                                                                        *
  *   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,  *
- *   2010, 2011, 2013, 2014 Free Software Foundation, Inc.                *
+ *   2010, 2011, 2013, 2014, 2015 Free Software Foundation, Inc.          *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 3, or (at your option)  *
@@ -26,6 +26,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
+#include <unistd.h>
 
 #ifdef HAVE_MAGIC_H
 #include <magic.h>
@@ -69,10 +70,8 @@ void set_colorpairs(void)
 		interface_color_pair[i].pairnum = A_NORMAL;
 	}
 
-	if (specified_color_combo[i] != NULL) {
-	    free(specified_color_combo[i]);
-	    specified_color_combo[i] = NULL;
-	}
+	free(specified_color_combo[i]);
+	specified_color_combo[i] = NULL;
     }
 
     for (; this_syntax != NULL; this_syntax = this_syntax->next) {
@@ -187,6 +186,21 @@ void color_update(void)
      * there was no syntax by that name, get the syntax based on the
      * file extension, then try the headerline, and then try magic. */
     if (openfile->colorstrings == NULL) {
+	char *currentdir = getcwd(NULL, PATH_MAX + 1);
+	char *joinednames = charalloc(PATH_MAX + 1);
+	char *fullname = NULL;
+
+	if (currentdir != NULL) {
+	    /* Concatenate the current working directory with the
+	     * specified filename, and canonicalize the result. */
+	    sprintf(joinednames, "%s/%s", currentdir, openfile->filename);
+	    fullname = realpath(joinednames, NULL);
+	    free(currentdir);
+	}
+
+	if (fullname == NULL)
+	    fullname = mallocstrcpy(fullname, openfile->filename);
+
 	for (tmpsyntax = syntaxes; tmpsyntax != NULL;
 		tmpsyntax = tmpsyntax->next) {
 
@@ -211,7 +225,7 @@ void color_update(void)
 		}
 
 		/* Set colorstrings if we match the extension regex. */
-		if (regexec(e->ext, openfile->filename, 0, NULL, 0) == 0) {
+		if (regexec(e->ext, fullname, 0, NULL, 0) == 0) {
 		    openfile->syntax = tmpsyntax;
 		    openfile->colorstrings = tmpsyntax->color;
 		    break;
@@ -224,6 +238,9 @@ void color_update(void)
 	    }
 	}
 
+	free(joinednames);
+	free(fullname);
+
 	/* Check the headerline if the extension didn't match anything. */
 	if (openfile->colorstrings == NULL) {
 #ifdef DEBUG
