@@ -1,4 +1,4 @@
-/* $Id: rcfile.c 5193 2015-04-12 11:15:57Z bens $ */
+/* $Id: rcfile.c 5265 2015-06-20 18:48:43Z bens $ */
 /**************************************************************************
  *   rcfile.c                                                             *
  *                                                                        *
@@ -42,16 +42,16 @@ static const rcoption rcopts[] = {
 #ifndef DISABLE_WRAPJUSTIFY
     {"fill", 0},
 #endif
-#ifndef NANO_TINY
-    {"locking", LOCKING},
+#ifndef DISABLE_HISTORIES
+    {"historylog", HISTORYLOG},
 #endif
+    {"morespace", MORE_SPACE},
 #ifndef DISABLE_MOUSE
     {"mouse", USE_MOUSE},
 #endif
 #ifndef DISABLE_MULTIBUFFER
     {"multibuffer", MULTIBUFFER},
 #endif
-    {"morespace", MORE_SPACE},
     {"nofollow", NOFOLLOW_SYMLINKS},
     {"nohelp", NO_HELP},
     {"nonewlines", NO_NEWLINES},
@@ -60,6 +60,9 @@ static const rcoption rcopts[] = {
 #endif
 #ifndef DISABLE_OPERATINGDIR
     {"operatingdir", 0},
+#endif
+#ifndef DISABLE_HISTORIES
+    {"poslog", POS_HISTORY},
 #endif
     {"preserve", PRESERVE},
 #ifndef DISABLE_JUSTIFY
@@ -79,25 +82,24 @@ static const rcoption rcopts[] = {
     {"tempfile", TEMP_FILE},
     {"view", VIEW_MODE},
 #ifndef NANO_TINY
+    {"allow_insecure_backup", INSECURE_BACKUP},
     {"autoindent", AUTOINDENT},
     {"backup", BACKUP_FILE},
-    {"allow_insecure_backup", INSECURE_BACKUP},
     {"backupdir", 0},
     {"backwards", BACKWARDS_SEARCH},
     {"casesensitive", CASE_SENSITIVE},
     {"cut", CUT_TO_END},
-    {"historylog", HISTORYLOG},
+    {"locking", LOCKING},
     {"matchbrackets", 0},
     {"noconvert", NO_CONVERT},
-    {"poslog", POS_HISTORY},
-    {"quiet", QUIET},
     {"quickblank", QUICK_BLANK},
+    {"quiet", QUIET},
     {"smarthome", SMART_HOME},
     {"smooth", SMOOTH_SCROLL},
+    {"softwrap", SOFTWRAP},
     {"tabstospaces", TABS_TO_SPACES},
     {"whitespace", 0},
     {"wordbounds", WORD_BOUNDS},
-    {"softwrap", SOFTWRAP},
 #endif
 #ifndef DISABLE_COLOR
     {"titlecolor", 0},
@@ -985,8 +987,7 @@ void parse_linter(char *ptr)
 	return;
     }
 
-    if (endsyntax->linter != NULL)
-	free(endsyntax->linter);
+    free(endsyntax->linter);
 
     /* Let them unset the linter by using "". */
     if (!strcmp(ptr, "\"\""))
@@ -1012,8 +1013,7 @@ void parse_formatter(char *ptr)
 	return;
     }
 
-    if (endsyntax->formatter != NULL)
-	free(endsyntax->formatter);
+    free(endsyntax->formatter);
 
     /* Let them unset the formatter by using "". */
     if (!strcmp(ptr, "\"\""))
