@@ -1,4 +1,4 @@
-/* $Id: search.c 5148 2015-03-22 11:42:29Z bens $ */
+/* $Id: search.c 5258 2015-06-20 08:10:25Z bens $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
@@ -106,6 +106,7 @@ void not_found_msg(const char *str)
 void search_replace_abort(void)
 {
     display_main_list();
+    focusing = FALSE;
 #ifndef NANO_TINY
     if (openfile->mark_set)
 	edit_refresh();
@@ -115,15 +116,6 @@ void search_replace_abort(void)
 #endif
 }
 
-/* Initialize the global search and replace strings. */
-void search_init_globals(void)
-{
-    if (last_search == NULL)
-	last_search = mallocstrcpy(NULL, "");
-    if (last_replace == NULL)
-	last_replace = mallocstrcpy(NULL, "");
-}
-
 /* Set up the system variables for a search or replace.  If use_answer
  * is TRUE, only set backupstring to answer.  Return -2 to run the
  * opposite program (search -> replace, replace -> search), return -1 if
@@ -155,7 +147,7 @@ int search_init(bool replacing, bool use_answer)
      * do_search() or do_replace() and be called again.  In that case,
      * we should put the same search string back up. */
 
-    search_init_globals();
+    focusing = TRUE;
 
     if (last_search[0] != '\0') {
 	char *disp = display_string(last_search, 0, COLS / 3, FALSE);
@@ -257,17 +249,16 @@ int search_init(bool replacing, bool use_answer)
     return 0;
 }
 
-/* Look for needle, starting at (current, current_x).  If no_sameline is
- * TRUE, skip over begin when looking for needle.  begin is the line
+/* Look for needle, starting at (current, current_x).  begin is the line
  * where we first started searching, at column begin_x.  The return
  * value specifies whether we found anything.  If we did, set needle_len
  * to the length of the string we found if it isn't NULL. */
 bool findnextstr(
 #ifndef DISABLE_SPELLER
-	bool whole_word,
+	bool whole_word_only,
 #endif
-	bool no_sameline, const filestruct *begin, size_t begin_x, const
-	char *needle, size_t *needle_len)
+	const filestruct *begin, size_t begin_x,
+	const char *needle, size_t *needle_len)
 {
     size_t found_len;
 	/* The length of the match we find. */
@@ -324,7 +315,7 @@ bool findnextstr(
 #ifndef DISABLE_SPELLER
 	    /* If we're searching for whole words, see if this potential
 	     * match is a whole word. */
-	    if (whole_word) {
+	    if (whole_word_only) {
 		char *word = mallocstrncpy(NULL, found, found_len + 1);
 		word[found_len] = '\0';
 
@@ -332,17 +323,11 @@ bool findnextstr(
 			fileptr->data, word);
 		free(word);
 	    }
-#endif
 
 	    /* If we're searching for whole words and this potential
-	     * match isn't a whole word, or if we're not allowed to find
-	     * a match on the same line we started on and this potential
-	     * match is on that line, continue searching. */
-	    if (
-#ifndef DISABLE_SPELLER
-		(!whole_word || found_whole) &&
+	     * match isn't a whole word, continue searching. */
+	    if (!whole_word_only || found_whole)
 #endif
-		(!no_sameline || fileptr != openfile->current))
 		break;
 	}
 
@@ -475,35 +460,13 @@ void do_search(void)
 #ifndef DISABLE_SPELLER
 	FALSE,
 #endif
-	FALSE, openfile->current, openfile->current_x, answer, NULL);
+	openfile->current, openfile->current_x, answer, NULL);
 
-    /* Check to see if there's only one occurrence of the string and
-     * we're on it now. */
+    /* If we found something, and we're back at the exact same spot where
+     * we started searching, then this is the only occurrence. */
     if (fileptr == openfile->current && fileptr_x ==
 	openfile->current_x && didfind) {
-#ifdef HAVE_REGEX_H
-	/* Do the search again, skipping over the current line, if we're
-	 * doing a bol and/or eol regex search ("^", "$", or "^$"), so
-	 * that we find one only once per line.  We should only end up
-	 * back at the same position if the string isn't found again, in
-	 * which case it's the only occurrence. */
-	if (ISSET(USE_REGEXP) && regexp_bol_or_eol(&search_regexp,
-		last_search)) {
-	    didfind = findnextstr(
-#ifndef DISABLE_SPELLER
-		FALSE,
-#endif
-		TRUE, openfile->current,
-		openfile->current_x, answer, NULL);
-	    if (fileptr == openfile->current && fileptr_x ==
-		openfile->current_x && !didfind)
-		statusbar(_("This is the only occurrence"));
-	} else {
-#endif
 	    statusbar(_("This is the only occurrence"));
-#ifdef HAVE_REGEX_H
-	}
-#endif
     }
 
     openfile->placewewant = xplustabs();
@@ -520,7 +483,14 @@ void do_research(void)
     size_t pww_save = openfile->placewewant;
     bool didfind;
 
-    search_init_globals();
+    focusing = TRUE;
+
+#ifndef DISABLE_HISTORIES
+    /* If nothing was searched for yet during this run of nano, but
+     * there is a search history, take the most recent item. */
+    if (last_search[0] == '\0' && searchbot->prev != NULL)
+	last_search = mallocstrcpy(last_search, searchbot->prev->data);
+#endif
 
     if (last_search[0] != '\0') {
 #ifdef HAVE_REGEX_H
@@ -534,36 +504,13 @@ void do_research(void)
 #ifndef DISABLE_SPELLER
 		FALSE,
 #endif
-		FALSE, openfile->current, openfile->current_x,
-		last_search, NULL);
+		openfile->current, openfile->current_x, last_search, NULL);
 
-	/* Check to see if there's only one occurrence of the string and
-	 * we're on it now. */
+	/* If we found something, and we're back at the exact same spot
+	 * where we started searching, then this is the only occurrence. */
 	if (fileptr == openfile->current && fileptr_x ==
 		openfile->current_x && didfind) {
-#ifdef HAVE_REGEX_H
-	    /* Do the search again, skipping over the current line, if
-	     * we're doing a bol and/or eol regex search ("^", "$", or
-	     * "^$"), so that we find one only once per line.  We should
-	     * only end up back at the same position if the string isn't
-	     * found again, in which case it's the only occurrence. */
-	    if (ISSET(USE_REGEXP) && regexp_bol_or_eol(&search_regexp,
-		last_search)) {
-		didfind = findnextstr(
-#ifndef DISABLE_SPELLER
-			FALSE,
-#endif
-			TRUE, openfile->current, openfile->current_x,
-			answer, NULL);
-		if (fileptr == openfile->current && fileptr_x ==
-			openfile->current_x && !didfind)
-		    statusbar(_("This is the only occurrence"));
-	    } else {
-#endif /* HAVE_REGEX_H */
 		statusbar(_("This is the only occurrence"));
-#ifdef HAVE_REGEX_H
-	    }
-#endif
 	}
     } else
 	statusbar(_("No current search pattern"));
@@ -676,7 +623,7 @@ char *replace_line(const char *needle)
  * canceled isn't NULL, set it to TRUE if we canceled. */
 ssize_t do_replace_loop(
 #ifndef DISABLE_SPELLER
-	bool whole_word,
+	bool whole_word_only,
 #endif
 	bool *canceled, const filestruct *real_current, size_t
 	*real_current_x, const char *needle)
@@ -684,31 +631,28 @@ ssize_t do_replace_loop(
     ssize_t numreplaced = -1;
     size_t match_len;
     bool replaceall = FALSE;
-#ifdef HAVE_REGEX_H
-    /* The starting-line match and bol/eol regex flags. */
-    bool begin_line = FALSE, bol_or_eol = FALSE;
-#endif
 #ifndef NANO_TINY
     bool old_mark_set = openfile->mark_set;
-    filestruct *edittop_save = openfile->edittop, *top, *bot;
+    filestruct *top, *bot;
     size_t top_x, bot_x;
     bool right_side_up = FALSE;
 	/* TRUE if (mark_begin, mark_begin_x) is the top of the mark,
 	 * FALSE if (current, current_x) is. */
 
     if (old_mark_set) {
-	/* If the mark is on, partition the filestruct so that it
-	 * contains only the marked text, set edittop to the top of the
-	 * partition, turn the mark off, and refresh the screen. */
+	/* If the mark is on, frame the region, and turn the mark off. */
 	mark_order((const filestruct **)&top, &top_x,
 	    (const filestruct **)&bot, &bot_x, &right_side_up);
-	filepart = partition_filestruct(top, top_x, bot, bot_x);
-	openfile->edittop = openfile->fileage;
 	openfile->mark_set = FALSE;
-#ifndef DISABLE_COLOR
-	reset_multis(openfile->current, TRUE);
-#endif
-	edit_refresh();
+
+	/* Start either at the top or the bottom of the marked region. */
+	if (!ISSET(BACKWARDS_SEARCH)) {
+	    openfile->current = top;
+	    openfile->current_x = (top_x == 0 ? 0 : top_x - 1);
+	} else {
+	    openfile->current = bot;
+	    openfile->current_x = bot_x;
+	}
     }
 #endif /* !NANO_TINY */
 
@@ -718,39 +662,23 @@ ssize_t do_replace_loop(
     findnextstr_wrap_reset();
     while (findnextstr(
 #ifndef DISABLE_SPELLER
-	whole_word,
+	whole_word_only,
 #endif
-#ifdef HAVE_REGEX_H
-	/* We should find a bol and/or eol regex only once per line.  If
-	 * the bol_or_eol flag is set, it means that the last search
-	 * found one on the beginning line, so we should skip over the
-	 * beginning line when doing this search. */
-	bol_or_eol
-#else
-	FALSE
-#endif
-	, real_current, *real_current_x, needle, &match_len)) {
+	real_current, *real_current_x, needle, &match_len)) {
 	int i = 0;
 
-#ifdef HAVE_REGEX_H
-	/* If the bol_or_eol flag is set, we've found a match on the
-	 * beginning line already, and we're still on the beginning line
-	 * after the search, it means that we've wrapped around, so
-	 * we're done. */
-	if (bol_or_eol && begin_line && openfile->current == real_current)
-	    break;
-	/* Otherwise, set the begin_line flag if we've found a match on
-	 * the beginning line, reset the bol_or_eol flag, and continue. */
-	else {
-	    if (openfile->current == real_current)
-		begin_line = TRUE;
-	    bol_or_eol = FALSE;
+#ifndef NANO_TINY
+	if (old_mark_set) {
+	    /* When we've found an occurrence outside of the marked region,
+	     * stop the fanfare. */
+	    if (openfile->current->lineno > bot->lineno ||
+		openfile->current->lineno < top->lineno ||
+		(openfile->current == bot && openfile->current_x > bot_x) ||
+		(openfile->current == top && openfile->current_x < top_x))
+		break;
 	}
 #endif
 
-	if (!replaceall)
-	    edit_refresh();
-
 	/* Indicate that we found the search string. */
 	if (numreplaced == -1)
 	    numreplaced = 0;
@@ -761,6 +689,8 @@ ssize_t do_replace_loop(
 		xpt, strnlenpt(openfile->current->data,
 		openfile->current_x + match_len) - xpt, FALSE);
 
+	    edit_refresh();
+
 	    curs_set(0);
 
 	    do_replace_highlight(TRUE, exp_word);
@@ -781,28 +711,19 @@ ssize_t do_replace_loop(
 	    }
 	}
 
-#ifdef HAVE_REGEX_H
-	/* Set the bol_or_eol flag if we're doing a bol and/or eol regex
-	 * replace ("^", "$", or "^$"). */
-	if (ISSET(USE_REGEXP) && regexp_bol_or_eol(&search_regexp,
-		needle))
-	    bol_or_eol = TRUE;
-#endif
-
 	if (i > 0 || replaceall) {	/* Yes, replace it!!!! */
 	    char *copy;
 	    size_t length_change;
 
 #ifndef NANO_TINY
-	    update_undo(REPLACE);
+	    add_undo(REPLACE);
 #endif
 	    if (i == 2)
 		replaceall = TRUE;
 
 	    copy = replace_line(needle);
 
-	    length_change = strlen(copy) -
-		strlen(openfile->current->data);
+	    length_change = strlen(copy) - strlen(openfile->current->data);
 
 #ifndef NANO_TINY
 	    /* If the mark was on and (mark_begin, mark_begin_x) was the
@@ -816,6 +737,7 @@ ssize_t do_replace_loop(
 			openfile->mark_begin_x = openfile->current_x;
 		    else
 			openfile->mark_begin_x += length_change;
+		    bot_x = openfile->mark_begin_x;
 		}
 	    }
 
@@ -826,15 +748,14 @@ ssize_t do_replace_loop(
 		/* Keep real_current_x in sync with the text changes. */
 		if (openfile->current == real_current &&
 			openfile->current_x <= *real_current_x) {
-		    if (*real_current_x <
-			openfile->current_x + match_len)
-			*real_current_x = openfile->current_x +
-				match_len;
+		    if (*real_current_x < openfile->current_x + match_len)
+			*real_current_x = openfile->current_x + match_len;
 		    *real_current_x += length_change;
-		}
 #ifndef NANO_TINY
-	    }
+		    bot_x = *real_current_x;
+		}
 #endif
+	    }
 
 	    /* Set the cursor at the last character of the replacement
 	     * text, so searching will resume after the replacement
@@ -852,9 +773,12 @@ ssize_t do_replace_loop(
 	    openfile->current->data = copy;
 
 #ifndef DISABLE_COLOR
-	reset_multis(openfile->current, TRUE);
+	    /* Reset the precalculated multiline-regex hints only when
+	     * the first replacement has been made. */
+	    if (numreplaced == 0)
+		reset_multis(openfile->current, TRUE);
 #endif
-	edit_refresh();
+
 	    if (!replaceall) {
 #ifndef DISABLE_COLOR
 		/* If color syntaxes are available and turned on, we
@@ -872,16 +796,12 @@ ssize_t do_replace_loop(
 	}
     }
 
+    if (numreplaced == -1)
+	not_found_msg(needle);
+
 #ifndef NANO_TINY
-    if (old_mark_set) {
-	/* If the mark was on, unpartition the filestruct so that it
-	 * contains all the text again, set edittop back to what it was
-	 * before, turn the mark back on, and refresh the screen. */
-	unpartition_filestruct(&filepart);
-	openfile->edittop = edittop_save;
+    if (old_mark_set)
 	openfile->mark_set = TRUE;
-	edit_refresh();
-    }
 #endif
 
     /* If the NO_NEWLINES flag isn't set, and text has been added to the
